package cn.ninegame.genericframework.module;

import android.content.Context;
import android.content.res.AssetManager;
import android.content.res.Resources;
import android.os.Build;

import java.io.File;
import java.lang.reflect.Method;
import java.util.HashMap;

import cn.ninegame.genericframework.basic.ExceptionManager;
import cn.ninegame.genericframework.basic.FrameworkService;
import cn.ninegame.genericframework.basic.StatManager;
import cn.ninegame.genericframework.config.ModulePathManager;
import cn.ninegame.genericframework.exception.DiskFullException;
import cn.ninegame.genericframework.exception.LoadContextException;
import cn.ninegame.genericframework.tools.ReflectHelper;

/**
 * Created by chenkai on 2016/1/19.
 */
public class ModuleContextLoader {
    private static final String TAG = "ModuleContextLoader";
    /**
     * For 5.0 and above, use the host APK's AssetManager directly.
     * For under 5.0, create a new AssetManager for all modules.
     */
    private static AssetManager getModuleAssetManager(Context base) throws LoadContextException, DiskFullException {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            return base.getAssets();
        } else {
            try {
                AssetManager am = AssetManager.class.newInstance();
                addAssetPath(am, base.getPackageResourcePath());
                return am;
            } catch (DiskFullException e) {
                throw e;
            } catch (Exception e) {
                throw new LoadContextException(e);
            }
        }
    }

    private static Resources genResourcesFromApk(Context base, String apkPath) throws LoadContextException, DiskFullException {
        Resources ori = base.getResources();
        Resources newRes;
        try {
            AssetManager am = getModuleAssetManager(base);
            addAssetPath(am, apkPath);
            newRes = new Resources(am, ori.getDisplayMetrics(), ori.getConfiguration());
        } catch (DiskFullException e) {
            throw e;
        } catch (Exception e) {
            throw new LoadContextException(e);
        }
        return newRes;
    }

    private static void addAssetPath(AssetManager am, String path) throws Exception {
        Method method = ReflectHelper.findMethod(AssetManager.class, "addAssetPath", new Class[]{String.class});
        method.setAccessible(true);
        int result = (Integer) method.invoke(am, path);
//        Log.d(TAG, "addAssetPath:" + result);
        if (result <= 0) {
            if (ExceptionManager.isDiskFullException()) {
                throw new DiskFullException("Failed to add asset path:" + path + ", result:" + result +", disk is full!");
            } else {
                throw new LoadContextException("Failed to add asset path:" + path + ", result:" + result);
            }
        }
    }

    /**
     * Generate module context from the module APK path.
     *
     * @param base Base context
     * @param info ModuleInfo
     * @param moduleDirType int
     * @return Context of the module APK
     * @throws LoadContextException
     */
    public static Context loadContext(Context base, ModuleInfo info, int moduleDirType) throws LoadContextException, DiskFullException {
        long start = System.currentTimeMillis();

        String id = info.getID();
        String version = info.getVersion();
        File apkFile ;

        if (ModuleLoader.DIR_LIB == moduleDirType) {
            apkFile = ModulePathManager.getInstance().getModuleApkInLib(id);
        } else {
            apkFile = ModulePathManager.getInstance().getModuleApk(id, version);
        }


        Resources newRes = genResourcesFromApk(base, apkFile.getAbsolutePath());
        if (newRes == null) {
            throw new LoadContextException("Could not generate Resources from " + apkFile);
        }
        Context result = new ModuleContext(base, newRes);

        long consume = System.currentTimeMillis() - start;
//        Log.i(TAG, "load context end, consume:" + consume);

        HashMap data = new HashMap<String, String>();
        data.put(StatManager.Constant.KEY_JANK_TYPE, StatManager.Constant.KEY_LOAD_MODULE_RES_JANK);
        data.put(StatManager.Constant.KEY_MODULE_ID, id);
        data.put(StatManager.Constant.KEY_JANK_INTERVAL, String.valueOf(consume));
        FrameworkService.getInstance().addStat(StatManager.Constant.CATEGORY_PERFORMANCE, data);

        return result;
    }
}
